use core::cell::RefCell;
use core::time::Duration;

use criterion::Criterion;

use imdb_async::*;

fn client_benches(c: &mut Criterion) {
	let rt = tokio::runtime::Builder::new_current_thread()
		.enable_time()
		.enable_io()
		.build()
		.unwrap();
	let mut group = c.benchmark_group("download");
	group.sample_size(10);
	//group.bench_function("download_titles", |b| b.to_async(&rt).iter(download_titles));
	//group.bench_function("download_episodes", |b| b.to_async(&rt).iter(download_episodes));
	group.finish();

	{
		let (_tmp, client) = rt.block_on(client());
		let client = RefCell::new(client);
		let _ = rt.block_on(lookup_single_title(&client));
		c.bench_function("lookup_single_title_cached", |b| b.to_async(&rt).iter(|| lookup_single_title(&client)));
	}

	{
		let (_tmp, client) = rt.block_on(client());
		let client = RefCell::new(client);
		rt.block_on(lookup_single_show(&client));
		c.bench_function("lookup_single_show_cached", |b| b.to_async(&rt).iter(|| lookup_single_show(&client)));
	}

	{
		let (_tmp, client) = rt.block_on(client());
		let client = RefCell::new(client);
		rt.block_on(lookup_several_shows(&client));
		c.bench_function("lookup_several_shows_cached", |b| b.to_async(&rt).iter(|| lookup_several_shows(&client)));
	}
}

criterion::criterion_group!(benches, client_benches);
criterion::criterion_main!(benches);

async fn client() -> (tempfile::TempDir, Client) {
	let dir = tempfile::TempDir::new().unwrap();
	let client = Client::new(Duration::from_secs(86400), dir.path()).await.unwrap();
	(dir, client)
}

pub async fn download_titles() {
	let (_tmp, mut client) = client().await;
	let _ = client.get_title(0).await;
}

pub async fn lookup_single_title(client: &RefCell<Client>) {
	let _ = client.borrow_mut().get_title(26972).await;
}

pub async fn lookup_single_show(client: &RefCell<Client>) {
	let _ = client.borrow_mut().get_show(460681).await;
}

pub async fn lookup_several_shows(client: &RefCell<Client>) {
	let _ = client
		.borrow_mut()
		.get_shows(&[118303, 369179, 898266, 1608180]).await;
}

